// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;

//blend properties
#define nRegions 3
float yBlends[nRegions-1];
float blendWidth=0.1;
float3 gammaRGB=1;
bool calibrate=0;
float lineWidth=0.01;


//texture projection properties
bool useProjection=false; // false=use vertex tex coords, true=use world coords
float4x4 tProjector;

//viewports
int ViewIndex: VIEWPORTINDEX;
int ViewCount: VIEWPORTCOUNT;
int3 ViewPortRemap = int3(0, 1, 2);

//samplers
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//only for the neares neighbour ps
sampler SampPoint = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = point;         //sampler states
    MinFilter = point;
    MagFilter = point;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//alpha
float Alpha <float uimin=0.0; float uimax=1.0;> = 1;

//include the bicubic texture lookup
#include "Bicubic.fxh"

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float4 TexCd : TEXCOORD0;
    float4 PosW : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0
    )
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    float4 PosW = mul(PosO, tW);

    //transform texturecoordinates
    if (useProjection)
    {
        Out.TexCd = mul(PosW, tProjector);
    }
    else
        Out.TexCd = mul(TexCd, tTex);
    
    Out.PosW = mul(PosO, tW);
    
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float3 factor(float pos)
{
    int ViewIndexMapped = ViewPortRemap[ViewIndex];
    float yTop = yBlends[ViewIndexMapped-1];
    float yBase = yBlends[ViewIndexMapped];


    float factortop = smoothstep(yTop-blendWidth/2,
                                 yTop+blendWidth/2,
                                 pos);
    if (ViewIndexMapped==0)
        factortop=1;


    float factorbottom = 1-smoothstep(yBase-blendWidth/2,
                                 yBase+blendWidth/2,
                                 pos);
    if (ViewIndexMapped==ViewCount-1)
        factorbottom=1;

    return pow(factortop * factorbottom, gammaRGB);
}

float4 calPattern(float4 PosW)
{
    int ViewIndexMapped = ViewPortRemap[ViewIndex];
    float yTop = yBlends[ViewIndexMapped-1];
    float yBase = yBlends[ViewIndexMapped];
    
    float4 col = 0;
    
    if (ViewIndexMapped>0)
        col += 1-smoothstep(lineWidth/2,lineWidth,abs(yTop-PosW.y));
        
    if (ViewIndexMapped<ViewCount-1)
        col += 1-smoothstep(lineWidth/2,lineWidth,abs(yBase-PosW.y));

// checkerboard
//    col.a = ((PosW.xyz/0.2) - floor(PosW.xyz/0.2)>0.5).x;

 return col;
}

// PS Bicubic
float4 PSbic(vs2ps In) : COLOR
{
  In.TexCd /= In.TexCd.w;
  In.TexCd.xy+=1;
  In.TexCd.xy /=2;
  In.TexCd.y=1-In.TexCd.y;

  float4 col = tex2Dbicubic(Samp, In.TexCd);
  
  col.a *= Alpha;
  
  col.rgb *= factor(In.PosW.y);
  
  if (calibrate)
     col += calPattern(In.PosW);
  
  return col;
}

// PS Bilinear
float4 PSlin(vs2ps In) : COLOR
{
  In.TexCd /= In.TexCd.w;
  In.TexCd.xy+=1;
  In.TexCd.xy /=2;
  In.TexCd.y=1-In.TexCd.y;
  float4 col = tex2D(Samp, In.TexCd);
  
  col.a *= Alpha;

  col.rgb *= factor(In.PosW.y);

  if (calibrate)
     col += calPattern(In.PosW);

  return col;
}

// PS Nearest Neighbour
float4 PSnn(vs2ps In) : COLOR
{
  float4 col = tex2D(SampPoint, In.TexCd);
  
  col.a *= Alpha;

  return col;
}



technique Bicubic
{
    pass p0
    {
		VertexShader = compile vs_1_1 VS();
		PixelShader = compile ps_3_0 PSbic();
    }
}

technique Bilinear
{
    pass p0
    {
		VertexShader = compile vs_1_1 VS();
		PixelShader = compile ps_2_0 PSlin();
    }
}

technique NearestNeighbour
{
    pass p0
    {
		VertexShader = compile vs_1_1 VS();
		PixelShader = compile ps_2_0 PSnn();
    }
}
